/**
/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/main', [
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/history/main',
    'com.voiceworks/ox-messenger/conversation/models/conversation',
    'com.voiceworks/ox-messenger/conversation/views/conversation',
    'com.voiceworks/ox-messenger/conversation/views/conversationlist'
], function (Aggregator, Session, Util, History, ConversationModel, ConversationView,
        ConversationListView) {
    'use strict';

    /**
     * Conversation Controller
     */
    var ConversationController = {

        init: function (app) {
            this.app = app;

            Aggregator.on('connection:reconnecting', $.proxy(this.reopenConversation, this));
            ox.on('relogin:success', $.proxy(this.reopenConversation, this));
            //ox.on('connection:online', $.proxy(this.reopenConversation, this));
        },

        reopenConversation: function () {
            var self = this;

            // save the jid/muc id for later use
            var conversationId = Session.user.conversations.currentConversationId;

            // reset the currentConversationId to force re-opening of the conversation View
            Session.user.conversations.currentConversationId = null;

            // When MUC history is loaded
            Aggregator.on('muchistory:loaded', function () {
                var newConversation = Session.user.conversations.get(conversationId);
                if (newConversation && ox.ui.App.getCurrentApp().get('name') === 'com.voiceworks/ox-messenger') {
                    self.showConversation(newConversation);
                }
            });

            // When XMPP history is loaded
            Aggregator.on('xmpp-history:loaded', function () {
                var newConversation = Session.user.conversations.get(conversationId);
                if (newConversation && ox.ui.App.getCurrentApp().get('name') === 'com.voiceworks/ox-messenger') {
                    self.showConversation(newConversation);
                }
            });

            Aggregator.trigger('focus:textarea');
        },

        initConversation: function ($conversationEl) {
            this.$conversationEl = $conversationEl;
        },

        initList: function ($listEl) {
            this.$listEl = $listEl;
        },

        showList: function () {
            var that = this;
            var oldSelection = (this.listView)?this.listView.selection.get():[];

            this.listView && this.listView.remove() && delete this.listView;
            this.listView = new ConversationListView({
                app: this.app,
                ignoreFocus: true,
                pagination: false,
                draggable: false
            });
            this.listView.setCollection(Session.user.conversations);
            this.$listEl.append(this.listView.render().el);
            this.listView.selection.set(oldSelection);
            this.listView.$el.off('mousedown.dnd');
            this.listView.on('selection:change', function (ids) {
                var conv;

                if (ids.length === 1){
                    conv = Session.user.conversations.get(ids[0]);
                } else {
                    conv = Session.user.conversations.get(Session.user.conversations.currentConversationId);
                    if (!conv && ids.length === 0 && Session.user.conversations.length > 0) {
                        conv = Session.user.conversations.first();
                    }
                }

                if(conv) {
                    that.showConversation(conv);
                }

                Aggregator.trigger('conversationlist:selection:change', ids);
            });

            return this;
        },

        conversationDeleted: function () {
            if (this.conversationView) {
                this.conversationView.remove() && delete this.conversationView;
            }
        },

        showConversation: function (conversation, forceCreate) {
            var self = this;

            History.hide();

            if (!conversation) {
                return;
            }

            // Reset the unread counter
            conversation.setUnreadMessages(0);

            // If we try to select the current conversation again & the View already exists
            if (!forceCreate &&
                Session.user.conversations.currentConversationId === conversation.id &&
                this.conversationView) {
                conversation.set('scroll', true);
            } else {
                Session.user.conversations.currentConversationId = conversation.id;

                if (this.conversationView) {
                    this.conversationView.remove();
                    this.conversationView = null;
                }

                this.conversationView = new ConversationView({
                    user: Session.user,
                    app: this.app,
                    model: conversation
                });

                // Container for showing the conversation
                var windowRightContainer = this.$conversationEl.find('.messenger--window-right-container');
                if (windowRightContainer.length > 0) {
                    windowRightContainer.children().remove();
                    windowRightContainer.prevObject = null;
                    windowRightContainer.append(this.conversationView.render().$el);
                } else {
                    windowRightContainer = $('<div class="messenger--window-right-container">');
                    windowRightContainer.append(this.conversationView.render().$el);
                }
                this.$conversationEl.append(windowRightContainer);

                // Compact View? Add correct class
                if (this.app.props.get('layout') === 'compact') {
                    windowRightContainer.addClass('messenger--window-right-container--compact');
                }

                this.conversationView.messageFormView.focusInput(true);
                this.conversationView.messageContainerCollectionView.scroll();

                // If true, we'll block temporarily (2") the user input
                if(conversation.delayUserInput) {
                    this.conversationView.messageFormView.delayUserInput(2000, function(){
                        conversation.delayUserInput = false;
                    });
                }

                // TODO This is so bad... Unify after paging for MUC is implemented
                if (!conversation.isMuc() && $('div.messenger--chat-messages').scrollTop() === 0
                    && conversation.messageArchives.length) {
                    this.preloadArchives(conversation);
                }

                // Paging
                $('div.messenger--chat-messages').scroll(function (e) {
                    if ($(this).scrollTop() === 0) {
                        // muc
                        if (conversation.isMuc()) {
                            conversation.getNextRsm();
                        }
                        // 1:1
                        else{
                            if (conversation.messageArchives.length) {
                                conversation.getNextArchiveMessage();
                            }
                        }

                        // Avoid autoScroll
                        self.conversationView.messageContainerCollectionView.userScrolledToTop = true;
                    } else {
                        self.conversationView.messageContainerCollectionView.userScrolledToTop = false;
                    }
                    // This event is always triggered despite user/code scroll, but messagecontainercollection.js
                    // switches 'userScrolled' to false when the scrolling's not been done by the user
                    self.conversationView.messageContainerCollectionView.userScrolled = true;
                });


                // Ajust height
                $(document).ready(function () {
                    if (self.conversationView) {
                        self.conversationView.setMessagingHeight();
                    }
                    $(window).on('resize', function () {
                        if (self.conversationView) {
                            self.conversationView.setMessagingHeight();
                        }
                    });
                });
            }
        },

        // TODO move the UI logic to the view
        preloadArchives: function (conversation) {
            var that = this;
            conversation.getNextArchiveMessage().then(function () {
                // Check if the conversation is still opened.
                var container = $('div.messenger--chat-messages');
                if (!container.length) {
                    return;
                }
                container.scrollTop(container[0].scrollHeight + 10);

                if (container.scrollTop() < 1 && conversation.messageArchives.length) {
                    that.preloadArchives(conversation);
                }
            });
        },

        delete: function (conversationId) {
            Session.user.conversations.remove(Session.user.conversations.get(conversationId));
            Session.user.conversations.trigger('reset');
            this.conversationDeleted();
        },

        start: function (participant, type) {
            var conversation = Session.user.conversations.get(participant.id);

            if (conversation) {
                return conversation;
            }

            conversation = new ConversationModel({
                id: participant.id,
                title: participant.getDisplayName(),
                user: Session.user,
                sortName: participant.getDisplayName(),
                timestamp: new Date().getTime()
            });

            conversation.addParticipant(participant);
            conversation.addParticipant(Session.user);
            Session.user.conversations.add(conversation);

            return conversation;
        }
    };

    return ConversationController;
});
