/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/models/conversation', [
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/roster/models/usercollection',
    'com.voiceworks/ox-messenger/conversation/models/messagecontainercollection',
    'com.voiceworks/ox-messenger/conversation/models/messagecontainer',
    'com.voiceworks/ox-messenger/conversation/models/messagecollection',
    'com.voiceworks/ox-messenger/conversation/models/message/factory',
    'com.voiceworks/ox-messenger/conversation/models/message/system',
    'com.voiceworks/ox-messenger/conversation/models/messagearchivecollection',
    'com.voiceworks/ox-messenger/conversation/models/messagearchive',
    'com.voiceworks/ox-messenger/conversation/models/messagelocalarchive',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/core/notification-control',
    'com.voiceworks/ox-messenger/core/notification',
    'com.voiceworks/ox-messenger/core/properties',
    'com.voiceworks/ox-messenger/core/vendor/promise',
    'com.voiceworks/ox-messenger/conversation/models/message',
    'com.voiceworks/ox-messenger/core/call',
    'settings!com.voiceworks/ox-messenger',
    'gettext!com.voiceworks/ox-messenger'
], function (Aggregator, RosterUserCollection, MessageGroupsCollection, MessageGroupModel, MessagesCollection,
             MessageFactory, SystemMessageModel, MessageArchiveCollection, MessageArchiveModel, MessageLocalArchiveModel, Util,
             NotificationControl, Notification, PropertiesModel, Promise, MessageModel, CallModel, settings, gt) {

    'use strict';

    /**
     *  Conversation
     *    Message Containers Collection
     *      Message Container
     *        Messages Collection
     *          Message
     *    Message Archive Collection
     */


    /**
     * Conversation Model
     */
    var ConversationModel = Backbone.Model.extend({

        defaults: {
            id: '',
            title: 'default chat',
            timestamp: -1,
            lastMsg: new MessageModel({message: '...', timestamp: -1}),
            previousMsgJid: false,
            callSession: null,
            callState: 'idle',
            unreadMessages: 0,
            scroll: false,
            timer: null,
            sortName: '',
            showNotifications: true,
            chatState: '',                  // contains the chat state ('paused', 'active', 'composing',..) for logged user
            composingTimeout: null,         // timeout of the logged user to manage the 'composing'/'paused' state
            composingParticipants: [],      // array with the participants of the chat that are typing (composing)
            whosComposingText: '',          // list of names of the users typing (composing)

            // Muc Paging (RSM)
            firstRsmId: -1,
            lastRsmId: -1,
            rsmCount: -1,
            totalCount: 0,
            waitingRsm: false,



            /**
             * Colleciton of archive collections, represented by the full jid and the start date.
             * @type {Array.<{jid: string, date: Date}>}
             */
            jidToDate: null
        },

        localHistorySize: 100,

        initialize: function (options) {
            this.user = options.user;
            this.participants = new RosterUserCollection();
            this.participants.sortFirst = this.user;
            this.callParticipants = new RosterUserCollection();
            this.messagesCollection = new MessagesCollection();
            this.messageGroups = new MessageGroupsCollection(
                null, null, this.messagesCollection);
            this.messageArchives = new MessageArchiveCollection();
            this.groupCallInvitees = new RosterUserCollection();
            this.chatState = '';
            this.composingTimeout = null;
            this.composingParticipants = [];
            this.whosComposingText = '';
            this.firstRsmId = -1;
            this.lastRsmId = -1;
            this.rsmCount = -1;
            this.totalCount = 0;
            this.waitingRsm = false;
            this.delayUserInput = false;

            this.set('jidToDate', []);
            this.set('showNotifications', true);
            this.set('totalCount', 0);

            this.properties = new PropertiesModel({
                groupMember: this.isMember(this.user),
                showNotifications: this.get('showNotifications')
            });

            this.participantLimit = {
                video: 5,
                audio: 20
            };

            Aggregator.on('conversation:showNotifications', $.proxy(this.handleShowNotifications, this));
        },

        /**
         * @param message
         * @TODO rename addMessage => handleMessage or something else?
         */
        addMessage: function (message) {
            var prevLength = this.messagesCollection.length;

            this.messagesCollection.add(message);
            if(this.messagesCollection.length > prevLength) {
                this.increaseTotalCount();
            }

            if (message.get('isOfflineMessage') || !message.get('isHistory') && !MessageFactory.isSystem(message)) {
                this.handleMessageNotificiations(this, message);
            }

            // Update the last message if it's not a System Message
            if (this.messagesCollection.length > 0){
                var lastMessage = this.messagesCollection.at(this.messagesCollection.length - 1);

                // TODO this.lastMessageHandler();
                if (!MessageFactory.isSystem(lastMessage)) {
                    this.set('lastMsg', lastMessage);
                    this.set('timestamp', lastMessage.get('timestamp'));
                }
            }
        },

        getNextArchiveMessage: function () {
            var archive = this.messageArchives.pop(),
                addMessage = this.addMessage.bind(this);

            return new Promise(function (resolve) {
                if (archive) {
                    archive.getMessageModels(function (msgs) {
                        msgs.forEach(function (item) {
                            addMessage(item);
                        });
                        resolve();
                    });
                } else {
                    resolve();
                }
            });
        },

        getNextRsm: function (limit) {
            limit = limit || 50;

            // Check if there's not a pending RSM request and there are still items to load...
            if(!this.get('waitingRsm') && !this.areAllMessagesLoaded()) {
                this.set('waitingRsm', true);
                Aggregator.trigger('groupconversation:next-rsm', {
                    conversationId: this.id,
                    firstMsgDbId: this.get('firstRsmId'),
                    limit: limit
                });
            }
        },

        setRsmData: function (first, last, count) {
            this.set('firstRsmId', first);
            this.set('lastRsmId', last);
            this.set('rsmCount', count);

            // Reset the Waiting RSM flag
            this.set('waitingRsm', false);
        },

        increaseTotalCount: function() {
            this.set('totalCount', this.get('totalCount') + 1);
        },

        addParticipant: function (participant, isHistory, avoidSystemMessage) {
            // Make sure the participant does not exist already in the conversation
            if (!this.existsBareParticipant(participant.id)) {
                var isLoggedUser = participant.id === this.user.id;
                this.participants.add(participant);
            }
        },

        removeParticipant: function (participant) {
            var isLoggedUser = participant.id === this.user.id;

            this.participants.remove(participant);

            // If the participant was properly deleted, create a system message
            if (!this.existsBareParticipant(participant.id)) {
                var sysMessage = new SystemMessageModel({
                    message: isLoggedUser?gt('You left the conversation') : gt('%s left the conversation', participant.getDisplayName()),
                    timestamp: new Date().getTime()
                });

                this.addMessage(sysMessage);
            }
        },

        getOtherParticipants: function () {
            var cloned;

            if (!this.user) {
                return this.participants;
            }

            // return without initiator
            cloned = this.participants.clone();
            cloned.models = _(this.participants.models).without(this.user);

            return cloned;
        },

        getParticipantsCount: function () {
            if (!this.participants) {
                return;
            }

            return this.participants.length;
        },

        getCallParticipantsCount: function () {
            if (!this.callParticipants) {
                return;
            }

            return this.callParticipants.length;
        },

        getParticipantsOnlineCount: function () {
            var onlineParticipants = this.participants.filter(function (p) {
                return p.isOnline() === true;
            });

            return onlineParticipants.length;
        },

        getParticipantsActiveCallCount: function () {
            var callParticipants = this.getCallParticipants();

            return callParticipants.length;
        },

        existsBareParticipant: function (jid) {
            var exists = false;

            this.participants.each(function (participant) {
                exists = exists || (Util.getBareJid(participant.id) === Util.getBareJid(jid));
            });

            return exists;
        },

        handleMessageNotificiations: function (conversation, message) {
            var unreadMessages = conversation.get('unreadMessages'),
                previousCount = unreadMessages ? unreadMessages : 0,
                newCount = 0,
                appActive = ox.ui.App.getCurrentApp().get('name') === 'com.voiceworks/ox-messenger';

            // Reset the count (= 0) when the conversation is the current open conversation
            if (appActive && conversation.user.conversations.currentConversationId === conversation.get('id')) {
                newCount = 0;

                // Don't add number if it's your own message
            } else if (appActive && conversation.user.get('id') === message.getSenderJid()) {
                newCount = unreadMessages;

                // Not current conversation and not your own message? Add 1 to the unreadMessages count
            } else {
                newCount = previousCount + 1;

                // Play sound for incoming message, unless the user is DND or if notifications are muted.
                NotificationControl.notify(conversation, {
                    sound: function () {
                        Util.playSound(settings.get('incomingMessageTone'), 'incomingMessagePing', false);
                    },
                    yell: function () {
                        Notification.yell(message.user, message);
                    }
                });
            }

            this.setUnreadMessages(newCount);

            if (this.collection) {
                this.collection.updateAppUnreadBadge();
            }
        },

        setUnreadMessages: function (newCount) {
            this.set('unreadMessages', newCount);

            this.updateLocalStorage(this.get('totalCount') - newCount);

            if (this.collection) {
                this.collection.updateAppUnreadBadge();
            }
        },

        isMuc: function () {
            return this.get('roomId') ? true : false;
        },

        createCallHistory: function (callHistory) {
            var that = this,
                messages = [];

            callHistory.each(function (cdr) {
                messages.push(that.createCallHistoryItem(cdr));
            });

            return messages;
        },

        createCallHistoryItem: function (cdr) {
            var message,
                duration = Util.getDurationFormatted(cdr.get('billmsec') / 1000);

            switch (cdr.getCallHangupCause()) {
                case 'cancelled':
                    message = gt('Cancelled call');
                    break;
                case 'declined':
                    //message = gt('Declined call');
                    message = gt('Not answered call');
                    break;
                case 'missed':
                    message = gt('Missed call');
                    break;
                default:
                    message = gt('Call - duration: %s', duration);
            }

            message = new SystemMessageModel({
                from: 'callhistory',
                message: message,
                timestamp: Date.parse(cdr.getStartStamp()),
                isHistory: false
            });

            return message;
        },

        hasActiveCall: function () {
            return this.getParticipantsActiveCallCount() > 1;
        },

        isMember: function (user) {
            return this.participants.get(user) ? true : false;
        },

        canGroupCall: function () {
            var that = this,
                errorMessages = {
                    video: [],
                    audio: [],
                    generic: []
                },
                isMember = this.isMember(this.user),
                callableParticipants = 0;

            /**
             * Generic Checks
             */
            if (!isMember) {
                errorMessages.generic.push(gt('You are not a member of the group chat.'));
            }

            if (this.getParticipantsCount() < 2) {
                errorMessages.generic.push(gt('You are the only member in the group chat.'));
            }

            if (this.user.call.isActive()) {
                errorMessages.generic.push(gt('You are already in a call.'));
            }

            if (this.user.get('mustFlash')) {
                errorMessages.generic.push('Calling is only supported using Chrome or Firefox.');
            }

            /**
             * Group Call Checks
             */
            var groupCallActive = false;
            var callType = this.get('callType') || 'audio';
            this.participants.each(function (participant) {
                if (participant === that.user) {
                    return;
                }

                var canCallWith = that.user.canCallWith(callType, participant, true);

                if (canCallWith === true) {
                    callableParticipants++;
                }

                if (participant.get('inGroupCall') && participant.get('inGroupCall') === that.get('roomId')) {
                    groupCallActive = true;
                }
            });

            /**
             * Checks when there is an active call in this conversation
             */
            if (this.hasActiveCall()) {

                var callParticipants = this.getCallParticipants();

                // Disable video buttons when maximum amount of users has been met.
                if (callParticipants.length >= this.participantLimit.video) {
                    errorMessages.video.push(gt('The maximum amount of %d active participants is reached.', this.participantLimit.video));
                }

                // Disable audio buttons when maximum amount of users has been met.
                if (callParticipants.length >= this.participantLimit.audio) {
                    errorMessages.audio.push(gt('The maximum amount of %d active participants is reached.', this.participantLimit.audio));
                }

                /**
                 * Checks when there is NOT an active call in this conversation
                 */
            } else {
                // Disable video buttons when maximum amount of users has been met.
                if (this.getParticipantsOnlineCount() > this.participantLimit.video) {
                    errorMessages.video.push(gt('You can not start a group video call with more than %d participants.', this.participantLimit.video));
                }

                // Disable audio buttons when maximum amount of users has been met.
                if (this.getParticipantsOnlineCount() > this.participantLimit.audio) {
                    errorMessages.audio.push(gt('You can not start a group audio call with more than %d participants.', this.participantLimit.audio));
                }

            }

            // Disable buttons when none of the participants is able to call (busy)
            if (callableParticipants === 0 && !groupCallActive && !this.user.get('mustFlash')) {
                errorMessages.generic.push(gt.ngettext(
                    'Group call not possible: a participant is busy or not able to call.',
                    'Group call not possible: all participants are busy or not able to call.',
                    this.participants - 1
                ));
            }

            errorMessages.audio = errorMessages.audio.concat(errorMessages.generic);
            errorMessages.video = errorMessages.video.concat(errorMessages.generic);

            if (errorMessages.audio.length || errorMessages.video.length) {
                return errorMessages;
            }

            return true;
        },

        canJoinGroupCall: function (callType) {
            var maxCallParticipants = (callType === 'groupvideo') ?  this.participantLimit.video : this.participantLimit.audio;
            var callParticipants = this.getCallParticipants();
            return callParticipants.length < maxCallParticipants;
        },

        areAllMessagesLoaded: function() {
          return this.messagesCollection.length === this.get('rsmCount');
        },

        sendGroupCallInvite: function (user, isOneToMany) {
            if (!this.groupCallInvitees.get(user)) {
                return false;
            }

            if(isOneToMany){
                //know who is the 3rd invitee to a new muc out of 1To1 call
                Aggregator.trigger('groupcall:sendInvite', {
                    conversation: this,
                    user: user,
                    notify: true,
                    invitedUser: true
                });
            } else {
                Aggregator.trigger('groupcall:sendInvite', {
                    conversation: this,
                    user: user,
                    notify: true
                });
            }

            this.groupCallInvitees.remove(user);

            return true;
        },

        // Local setting for conversation to mute notifications
        isMuted: function () {
            return !this.get('showNotifications');
        },

        handleShowNotifications: function (object) {
            if (this.cid !== object.conversation.cid) {
                return;
            }

            this.set('showNotifications', object.showChatNotifications);
        },

        handleLastPage: function(val) {

        },

        // Returns the number of participants that are 'ready' on the conversation
        getNumParticipantsReady: function () {
            return this.participants
                .filter(function (participant) {return participant.call && participant.call.get('readyReceived'); })
                .length;
        },

        // Returns the number of participants that are 'ready' on the conversation
        getNumParticipantsInGroupCall: function () {
            var thisRoomId = this.get('roomId');

            return this.participants
                .filter(function (participant) {
                    return (participant.get('inGroupCall') && participant.get('inGroupCall') === thisRoomId);
                }).length;
        },

        // Starts a activity timeout that will callback whether there's activity (anybody 'ready') or not in this
        // conversation right after 60s
        startActivityTimeout: function (cb) {
            var conversation = this,
                timeoutTime = 60000;

            setTimeout(function () {
                cb(conversation.getNumParticipantsReady() > 1);
            }, timeoutTime);
        },

        addComposingParticipant: function(user){
            // Make sure we don't duplicate the participant
            this.removeComposingParticipant(user);

            // Add participant
            this.composingParticipants.push(user);

            this.updateWhoisComposing();
        },

        removeComposingParticipant: function(user){
            // Remove participant
            if(this.composingParticipants.indexOf(user) >= 0){
                this.composingParticipants.splice(this.composingParticipants.indexOf(user), 1);
            }

            this.updateWhoisComposing();
        },

        setChatState: function (state) {
            var trigger = true;

            // Assign the new state
            this.chatState = state;

            // Timmers & Triggers
            if(state === ConversationModel.STATE_ACTIVE){
                this.clearChatStateTimeout();
                // We never trigger ACTIVE states with no body
                trigger = false;
            } else if(state === ConversationModel.STATE_PAUSED){
                this.clearChatStateTimeout();
            } else if(state === ConversationModel.STATE_COMPOSING){
                // If the timeout was not in progress, we'll trigger the event
                if(this.resetComposingTimeout()){
                    trigger = false;
                }
            }

            // Trigger the state change
            if(trigger){
                Aggregator.trigger('conversation:state', {
                    conversation: this,
                    state: state
                });
            }
        },

        // Provided a fileDesc, it returns the first and (hopefully) only message with that file desc
        getMessageByFileDesc: function(fileDesc) {
            var theMessage = _.find(this.messagesCollection.models, function(message){
                return (message.has('fileDesc') && message.get('fileDesc').cid === fileDesc.cid);
            });

            return theMessage;
        },

        // Resets the timer to trigger 'conversation:state' paused message
        resetComposingTimeout: function() {
            var timeoutCleared,
                onTimeout = function(){
                    // Dispatch a 'paused' event
                    this.setChatState(ConversationModel.STATE_PAUSED);
                    clearTimeout(this.composingTimeout);
                };

            timeoutCleared = this.clearChatStateTimeout();
            this.composingTimeout = setTimeout($.proxy(onTimeout, this), ConversationModel.STATE_PAUSED_TIME);

            return timeoutCleared;
        },

        clearChatStateTimeout: function () {
            if(this.composingTimeout){
                clearTimeout(this.composingTimeout);
                this.composingTimeout = null;
                return true;
            }
            return false;
        },

        /**
         * TODO move lang strings to the lang file
         */
        updateWhoisComposing: function() {
            var text = '';

            if(this.composingParticipants.length === 1){
                text = gt('%s is typing...',
                    this.composingParticipants[0].getDisplayName());
            }
            else if(this.composingParticipants.length === 2){
                text = gt('%s and %s are typing...',
                    this.composingParticipants[0].getDisplayName(),
                    this.composingParticipants[1].getDisplayName());
            }
            else if(this.composingParticipants.length > 0){
                // TODO show the number of others
                text = gt('%s, %s and # others are typing...',
                    this.composingParticipants[0].getDisplayName(),
                    this.composingParticipants[1].getDisplayName());
            }

            this.set('whosComposingText', text);
        },
        isMarkedForDeletion: function() {
            if (typeof this.get('jidToDate').forDelete === 'undefined') {
                return false;
            }
            else {
                return true;
            }
        },

        updateLocalStorage: function(count) {
            var lastCount = this.getLocalStorage();

            if(!Number.isNaN(lastCount)) {
                lastCount = Math.max(lastCount, count);
            } else {
                lastCount = count;
            }

            localStorage.setItem(this.get('id'), lastCount);
        },

        getLocalStorage: function () {
            var lastCount = localStorage.getItem(this.get('id'));
            return parseInt(lastCount);
        },

        addCallParticipant: function (jid, callSession, readyReceived, callType) {
            var participant = this.participants.get(jid);

            if (!participant) {
                return false;
            }

            if (!participant.call) {
                var call = new CallModel({
                    callSession: callSession,
                    readyReceived: readyReceived,
                    type: callType,
                    conversationId: this.id
                });

                participant.call = call;
            } else {
                participant.call.set({
                    callSession: callSession,
                    readyReceived: readyReceived,
                    type: callType,
                    conversationId: this.id
                });
            }

            this.callParticipants.add(participant);

            return true;
        },

        getCallParticipants: function () {
            var self = this;
            var callParticipants = this.participants.filter(function (p) {
                return p.get('inGroupCall')  === self.get('roomId');
            });

            return callParticipants;
        },

    });

    // ms to consider the chat must be switched to PAUSED
    ConversationModel.STATE_PAUSED_TIME = 15000;

    // Chat states (per user)
    ConversationModel.STATE_PAUSED      = 'paused';
    ConversationModel.STATE_ACTIVE      = 'active';
    ConversationModel.STATE_COMPOSING   = 'composing';

    return ConversationModel;
});
