/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/messagecontainercollection', [
    'io.ox/core/extensions',
    'com.voiceworks/ox-messenger/conversation/models/message/factory',
    'com.voiceworks/ox-messenger/core/views/virtualdom',
    'com.voiceworks/ox-messenger/conversation/views/messagecontainer_chat',
    'com.voiceworks/ox-messenger/conversation/views/messagecontainer_system',
    'com.voiceworks/ox-messenger/conversation/views/messagecontainer_file'
], function (ext, MessageFactory, VirtualDomView, ChatMessageContainerView,
        SystemMessageContainerView, FileMessageContainerView) {
    'use strict';

    /**
     * Base Message Container View
     * this.collection = messageContainer Models
     *
     * TODO this should be a collection view
     */
    var MessageContainerCollectionView = VirtualDomView.extend({

        klassName: 'MessageContainerCollectionView',
        className: 'messenger--table__row messenger--table__row--chat-messages',
        messagesCollectionNode: null,

        /**
         * Map of views for message groups. indexed by their IDs.
         */
        views: {},

        initialize: function (options) {
            MessageContainerCollectionView.__super__.initialize.apply(this, arguments);

            // Init variables
            this.conversation = options.conversation;

            // Flag to mark user scrolled manually to the TOP
            this.userScrolledToTop = false;
            this.userScrolled = false;

            this.listenTo(this.collection, 'add', function(model) {
                this.createViewForMsgGroupModel(model);
            });
            this.collection.forEach(this.createViewForMsgGroupModel.bind(this));

            // Listen directly on the flag msgs collection
            this.listenTo(this.collection.messagesCollection, 'add', _.debounce(function () {
                if (this._vDomElement) {
                    this.render();
                }
            }));
        },

        createViewForMsgGroupModel: function(model) {
            if (this.views[model.cid]) {
                return this.views[model.cid];
            }

            var firstMessage = model.messages.first(),
                view;

            // if it's an file message
            if(firstMessage.get('fileDesc'))
                view = new FileMessageContainerView({
                    model: model
                });

            // is it System
            else if (MessageFactory.isSystem(firstMessage)) {
                view = new SystemMessageContainerView({
                    model: model
                });

                // else it's Chat or Group
            } else {
                view = new ChatMessageContainerView({
                    model: model
                });
            }

            this.views[model.cid] = view;
            return view;
        },

        dispose: function() {
            Object.keys(this.views).forEach(function(key) {
                this.views[key].dispose();
                delete this.views[key];
            }, this);
        },

        getMessagesCollectionNode: function () {
            if (!this.messagesCollectionNode || !this.messagesCollectionNode.length) {
                this.messagesCollectionNode = $('div.messenger--chat-messages');
            }

            return this.messagesCollectionNode;
        },

        template: function () {
            var h = this.h;

            // TODO loose the const reference (not needed)
            if (!this.cache) {
                this.cache = [];
            } else {
                this.cache.length = 0;
            }

            this.collection.forEach(function(model) {
                var view = this.createViewForMsgGroupModel(model);
                var cache = view.getCache();
                var template;

                if (!cache) {
                    template = view.template();
                    template.key = model.cid;
                }
                this.cache.push(cache || template);
            }, this);

            return h('div.messenger--table__cell.messenger--table__cell--chat-messages', [
                h('div.messenger--chat-messages-wrapper', [
                    h('div.messenger--chat-messages', this.cache)
                ])
            ]);
        },

        render: function () {
            var ret = MessageContainerCollectionView.__super__.render.call(this);
            var self = this;

            // Support for file view/download popup on mouse position
            this.$el.find('.messenger--chat-message__file.unselectable.dropup').on('click', function(e){
                var tempX            = e.offsetX,
                    tempY            = e.offsetY,
                    apsoluteMouseY   = $(e.target).parents('.messenger--media__body').position().top + tempY,
                    ySpaceForPopup   = $(e.target).parents('.messenger--chat-messages-wrapper').height() - apsoluteMouseY;

                if ( ySpaceForPopup < self.$el.find('.messenger--chat-message__file.unselectable.dropup .dropdown-menu').height() ) {
                    tempY = tempY - self.$el.find('.messenger--chat-message__file.unselectable.dropup .dropdown-menu').height();
                }

                self.$el.find('.messenger--chat-message__file.unselectable.dropup .dropdown-menu').css({ 'top': tempY, 'left': tempX });
            });

            // Force the code to be executed on the next JS cycle
            setTimeout(function() {
                // If user scrolled manually to the top, we'll request one more page,
                // so we need to change the scroll position after rendering so user can continue scrolling up
                // for more pages
                if (self.userScrolledToTop) {
                    self.scroll(100);
                } else {
                    self.scroll();
                }

                // Make sure we load enough pages to fill up all the vertical space (if enough messages)
                if(self.conversation.isMuc()) {
                    if(!self.conversation.areAllMessagesLoaded()) {
                        var messagesHeight = 0;

                        $('.messenger--chat-messages').children().each(function(){
                            messagesHeight = messagesHeight + $(this).outerHeight(true);
                        });

                        if(messagesHeight < $('.messenger--chat-messages-wrapper').height()) {
                            self.conversation.getNextRsm();
                        }
                    }
                }
            }, 0);

            return ret;
        },

        isScrollOnBottom: function() {
            var node = this.getMessagesCollectionNode();

            if (node.length) {
                return (node[0].scrollHeight - node[0].offsetHeight) === node.scrollTop();
            }
            return false;
        },

        scroll: function (posY) {
            var node = this.getMessagesCollectionNode();
            var self = this;

            // Define the scroll event only once
            function onScroll() {
                self.userScrolled = false;
                node.off('scroll', onScroll);
            }

            // Scroll to the assigned positon
            if (node.length) {
                if(posY !== undefined) {
                    node.scrollTop(posY);
                } else {
                    node.scrollTop(node[0].scrollHeight + 10);
                }

                // Listen to the scroll event to switch the 'userScrolled' Flags to false
                node.on('scroll', onScroll);
            }
        },
    });

    return MessageContainerCollectionView;
});
