define('com.voiceworks/ox-messenger/core/notification-control', [
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/core/session',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger'
], function (Aggregator, Util, Session, gt, Settings) {

    'use strict';

    var NotificationControl = {

        /**
         * Trigger Notifications
         * @param  {Model}   conversation  the conversation model
         * @param  {Object}  functions     Object with functions to perform
         * @return {null}                  Object with notification to play
         */
        notify: function (conversation, functions) {
            var settings = this.check(conversation);

            if (settings.desktop && typeof functions.desktop === 'function') {
                functions.desktop();
            }

            if (settings.sound && typeof functions.sound === 'function') {
                functions.sound();
            }

            if (settings.yell && typeof functions.yell === 'function') {
                functions.yell();
            }

            if (settings.call && typeof functions.call === 'function') {
                functions.call();
            }
        },

        /**
         * Check
         * @param  {Model} conversation  the conversation model
         * @return {Object}              the mute settings object
         */
        check: function (conversation) {
            var settings = {
                desktop: false,
                sound: false,
                yell: false,
                call: true
            };

            // Return the muted state when converation is not there
            if (conversation === null || typeof conversation === 'undefined') {
                return settings;
            }

            // Return the muted state when all notifications are muted in the conversation
            if (conversation.isMuted()) {
                return settings;
            }

            // Return the mute states for the different notification systems
            settings = {
                desktop: this.getDesktopSettings(conversation.user),
                sound: this.getSoundsSettings(conversation.user),
                yell: this.getYellSettings(),
                call: this.getCallSettings()
            };

            return settings;
        },

        /**
         * Get Desktop settings
         * @return {Bool}         true to show desktop notification, false to mute desktop notifications
         */
        getDesktopSettings: function (user) {
            if (user && user.isDnd()) {
                return false;
            }

            return Settings.get('showDesktopNotifications');
        },

        /**
         * Get Sound settings
         * @return {Bool}         true to play sounds, false to mute sounds
         */
        getSoundsSettings: function (user) {
            if (user && user.isDnd()) {
                return false;
            }

            if (Settings.get('incomingMessageTone') === '' && Settings.get('incomingCallRingtone') === '') {
                return false;
            }

            return true;
        },

        /**
         * Get Yell settings
         * @return {Bool}         true to show yell notification, false to mute yell notification
         */
        getYellSettings: function () {
            return Settings.get('showChatNotifications');
        },

        /**
         * Get Call settings
         * @return {Bool}         true to allow call, false to ignore/disallow call
         */
        getCallSettings: function () {
            // for now call is always allowed
            return true;
        },

    };

    return NotificationControl;
});