/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/sessionuser', [
    'io.ox/core/capabilities',
    'com.voiceworks/ox-messenger/core/user',
    'com.voiceworks/ox-messenger/core/call',
    'com.voiceworks/ox-messenger/conversation/models/conversationcollection',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/history/models/historycollection',
    'gettext!com.voiceworks/ox-messenger'
], function (Capabilities, UserModel, CallModel, ConversationCollection, Aggregator, HistoryCollection, gt) {
    'use strict';


    var SessionUserModel = UserModel.extend({
        callActive: false,
        callRinging: false,

        initialize: function (options) {
            SessionUserModel.__super__.initialize.apply(this, arguments);

            this.contacts = options ? options.contacts : null;
            this.conversations = new ConversationCollection();
            this.call = new CallModel();
            this.callHistory = new HistoryCollection();

            this.set('rosterQuery', '');

            this.listenTo(this.conversations, 'add remove', this.setupConversationSettings);
        },

        isCallable: function () {
            return !(this.callActive || this.callRinging);
        },


        setStatus: function (newStatus, statusMessage) {
            newStatus = newStatus.toLowerCase();

            // Map unknown statuses to 'online'
            if (_.indexOf(this.allowedStatusses, newStatus) === -1) {
                newStatus = 'online';
            }

            this.set({
                status: newStatus,
                statusMessage: statusMessage
            });
        },

        setInCall: function (callActive, conversation, message) {
            this.callActive = callActive;

            if (callActive) {
                this.set('prev_status', this.get('status'));
                // Do not use gt here: this is a system message
                //this.setStatus('xa', 'In a call');
                Aggregator.trigger('session:status', {status: 'xa', statusMessage: 'In a call'});
                Aggregator.trigger('call:started');
            } else {
                if (this.get('prev_status')) {
                    //this.setStatus(this.get('prev_status'));
                    if (message) {
                        Aggregator.trigger('session:status', {status: this.get('prev_status'), statusMessage: message});
                    }
                    else {
                        Aggregator.trigger('session:status', {status: this.get('prev_status'), statusMessage: ''});
                    }
                }
                Aggregator.trigger('call:ended');
            }

            Aggregator.trigger('call:current', { conversation: conversation });
        },

        _otherUserIsInCall: function (user) {
            var status = user.get('status').toLowerCase(),
                statusMessage = user.get('statusMessage');

            if (status == 'xa' && statusMessage) {
                if (statusMessage.toLowerCase() === 'in a call') {
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        },

        canDo: function (type, user) {
            var typeUp = type.charAt(0).toUpperCase() + type.slice(1),
                method = 'can' + typeUp + 'With';

            return this[method](user);
        },

        _checkBothOnline: function (user, err) {
            if (this.isOffline()) {
                err.push(gt('You can not (video) call, you seem to be offline.'));
            }

            if (user && user.isOffline()) {
                try {
                    err.push(gt('%s is not online and therefore not able to (video) call.', user.getDisplayName()));
                } catch (e) {
                    err.push(gt('This user is not online and therefore not able to (video) call.'));
                }
            }

            return err.length > 0 ? false : true;
        },

        canChatWith: function (user) {
            var err = [];
            if (user === undefined) {
                err.push(gt('User not found'));
            }
            // @todo: when offline messaging is possible, change this check
            //this._checkBothOnline(user, err);
            return err.length > 0 ? err : true;
        },

        canCallWith: function (type, user, isGroupCall) {
            type = type.replace('group', '');

            var err = [],
                typeUp = type.charAt(0).toUpperCase() + type.slice(1),
                canDo = 'can' + typeUp,
                capability = isGroupCall ? 'messenger-group-call' : 'messenger-call';

            if (!this._checkBothOnline(user, err)) {
                return err;
            }

            if (!Capabilities.has(capability)) {
                err.push(gt('You don\'t have %s capability.', capability));
            }
            if (!this.get(canDo)) {
                err.push(gt('Your browser does not suport %s, or you have no %s', type, (type === 'audio') ? 'microphone' : 'webcam'));
            }
            if (!user.get(canDo)) {
                err.push(gt('The browser of %s does not support %s or has no %s.', user.getDisplayName(), type, (type === 'audio') ? 'microphone' : 'webcam'));
            }
            if (!isGroupCall && this.call.isActive()) {
                err.push(gt('You are already in a call.'));
            }
            if (this._otherUserIsInCall(user)) {
                err.push(gt('%s is in a call and is not available.', user.getDisplayName()));
            }

            // Flash checks
            // @TODO-FLASH what's happening here?
            if (this.get('mustFlash')) {
                err.push(gt('Calling is only supported using Chrome or Firefox.'));
            } else if (user.get('mustFlash')) {
                if (isGroupCall) {
                    err.push(gt('Calling is only supported using Chrome or Firefox. Please ask %s to use Chrome or Firefox.', user.getDisplayName()));
                } else {
                    err.push(gt('Calling is only supported using Chrome or Firefox. Please ask %s to use Chrome or Firefox.', user.getDisplayName()));
                }
            }

            return err.length > 0 ? err : true;
        },

        canAudioWith: function (user) {
            return this.canCallWith('audio', user);
        },

        canVideoWith: function (user) {
            return this.canCallWith('video', user);
        },

        getCurrentCallConversation: function () {
            if (!this.call || this.conversations.length < 1) {
                return;
            }

            var activeCallConversationId = this.call.get('conversationId'),
                activeCallConversation = this.conversations.get(activeCallConversationId);

            return activeCallConversation;
        },

        setupConversationSettings: function (e) {
            Aggregator.trigger('conversation:setupNotifications', e);
        },

        endCall: function () {
            if (this.call) {
                this.call.end();
            }
            this.callRinging = false;
            this.callActive = false;
        },

    });

    return SessionUserModel;
});
