/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/user', [
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/core/properties',
    'settings!com.voiceworks/ox-messenger',
    'gettext!com.voiceworks/ox-messenger'
], function (Util, PropertiesModel, settings, gt) {
    'use strict';

    /**
     * User Model
     */
    var UserModel = Backbone.Model.extend({

        defaults: {
            id: null,
            name: null,
            subscription: null,
            ask: null,
            status: 'offline',
            nickname: null,
            isInvisible: false,
            statusMessage: null,
            canChat: false,
            canAudio: false,
            canVideo: false,
            hasFlash: false,
            mustFlash: false,
            contact: {}
        },

        allowedStatusses: ['online', 'away', 'dnd', 'xa', 'invisible', 'offline'],

        initialize: function () {
            this.groups = new Backbone.Collection();
            this.properties = new PropertiesModel({
                1 : true,  // Buddylist
                2 : false, // Favorites
                3 : false, // Guest Users
            });
            // Custom Groups take ID's > 3
        },


        setStatus: function (newStatus, statusMessage) {
            newStatus = newStatus.toLowerCase();

            // Map unknown statuses to 'online'
            if (_.indexOf(this.allowedStatusses, newStatus) === -1) {
                newStatus = 'online';
            }

            // Map invisible users to  status 'offline'
            if (newStatus === 'invisible') {
                this.set('isInvisible', true);
                newStatus = 'offline';
            } else {
                this.set('isInvisible', false);
            }

            this.set({
                status: newStatus,
                statusMessage: statusMessage
            });
        },

        isOnline: function () {
            return this.get('status') === 'online';
        },

        isOffline: function () {
            return this.get('status') === 'offline';
        },

        getImageUrl: function (size) {
            var imgUrl = this.get('contact').image1_url,
                rand = Math.floor((Math.random() * 1000000));

            if (!imgUrl) {
                return ox.base + '/apps/themes/default/fallback-image-contact.png';
            }

            switch (size) {
            case 'small':
                return Util.appendQuery(imgUrl, Util.imageOptionsSmall);
            default:
            case 'large':
                return Util.appendQuery(imgUrl, Util.imageOptions);
            }
        },

        setImageUrl: function (imgUrl) {
            this.get('contact').image1_url = imgUrl;
            this.set('image1_url', imgUrl);
        },

        setFullName: function (firstName, lastName) {
            this.get('contact').first_name    = firstName;
            this.get('contact').last_name     = lastName;
            this.get('contact').display_name  = Util.composeDisplayName(firstName, lastName, this.id);
        },

        getDisplayName: function () {
            var contact = this.get('contact');

            if (contact.display_name) {
                return contact.display_name;
            }
            return Util.composeDisplayName(contact.first_name, contact.last_name, this.id);
        },

        // @TODO: change status immediately after 'set', do not store 'incorrect statuses'
        getDisplayStatus: function () {
            var status = this.get('status').toLowerCase(),
                statusMessage = this.get('statusMessage');

            if (status == 'xa' && statusMessage) {
                switch (statusMessage.toLowerCase()) {
                case 'in a call':
                    //#. Presence message "In a call"
                    return gt('In a call');
                default:
                    // Unsupported status message, show status instead
                }
            }

            switch (status) {
            case 'online':
                //#. Presence status "Online"
                return gt('Online');
            case 'away':
                //#. Presence status "Away"
                return gt('Away');
            case 'dnd':
                //#. Presence status "Do Not Disturb"
                return gt('DND');
            case 'xa':
                //#. Presence status "Extended Away"
                return gt('XA');
            case 'invisible':
                //#. Presence status "Invisible"
                return gt('Invisible');
            case 'offline':
                //#. Presence status "Offline"
                return gt('Offline');
            default:
                //#. Presence status "Unknown"
                return gt('Unknown');
            }
        },

        // Returns the css selector to get the user's DOM elements
        getDomFilter: function () {
            return '[jid=\'' + this.id + '\']';
        },

        // Is User available to disturb/call/notify?
        isDnd: function () {
            return this.get('status').toLowerCase() === 'dnd';
        },

        isGuest: function () {
            return this.properties.toArray().indexOf(3) !== -1;
        },

        isFavorited: function () {
            return this.properties.toArray().indexOf(2) !== -1;
        },

        isSubscribed: function () {
            return this.get('subscription') === 'both';
        },

        setContact: function(contactData) {
            contactData.id = contactData.user_id;
            this.set('contact', contactData);
        },

        getStreamId: function() {
            return 'stream-' + this.id.replace(/[@|.]/g, '_');
        }

    });

    return UserModel;
});
