/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/core/views/dropdown', [
    'com.voiceworks/ox-messenger/aggregator',
    'io.ox/backbone/mini-views/dropdown',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/core/properties',
    'com.voiceworks/ox-messenger/core/vendor/offscreen',
    'gettext!com.voiceworks/ox-messenger'
], function (Aggregator, CoreDropdown, DisposableView, PropertiesModel, Offscreen, gt) {
    'use strict';

    var Dropdown = DisposableView.extend({

        className: 'dropdown',

        events: {
            'click .dropdown-toggle': 'handleToggle',
            'click .dropdown-item': 'handleClick'
        },

        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);

            var o = this.options = options || {};

            this.model = this.model || new PropertiesModel();
            this.extraModels = {};

            if (this.model) {
                this.listenTo(this.model, 'change', this.update);
            }

            this.tree = {};

            this.$ul = $('<ul class="dropdown-menu" role="menu">');
            this.toggleElement = $('<a id="dLabel" href="#" class="dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">');

            if (!o) {
                return;
            }

            this.cssClasses    = o.cssClasses ? o.cssClasses : false;
            this.extraModels   = o.extraModels ? o.extraModels : {};
            this.toggleElement = o.toggleElement ? o.toggleElement : this.toggleElement;
            this.toggleIcon    = o.toggleIcon ? o.toggleIcon : false;
            this.toggleLabel   = gt('Open Dropdown');

            if (!o.toggleLabel || o.toggleLabel.length === 0) {
                this.toggleLabel = '';
            }
            else if (o.toggleLabel) {
                this.toggleLabel = o.toggleLabel;
            }

            _.bindAll(this, 'render', 'beforeRender', 'afterRender');
            this.render = _.wrap(this.render, function (render) {
                this.beforeRender();
                render();
                this.afterRender();
                return this;
            });
        },

        /**
         * handleClick - Handle items clicked in the dropdown
         */
        handleClick: function (event) {
            event.preventDefault();
            event.stopPropagation();

            var data = $.extend({}, $(event.currentTarget).data());

            this.model.set(data.name, data.value);

            Aggregator.trigger('dropdown:item', {
                model: this.model,
                extraModels: this.extraModels,
                event: event,
                name: data.name,
                value: data.value
            });
        },

        /**
         * handleToggle - Hook to do stuff when dropdown is opened
         */
        handleToggle: function (event) {

            // Prevent bubbling to disable the Bootstrap Dropdown open logic
            event.preventDefault();
            event.stopPropagation();
            this.$el.toggleClass('open');

            // Call to afterRender to toggle the offscreen()
            this.afterRender();
        },

        /**
         * Render
         */
        render: function () {
            this.$el.children().remove();
            this.$el.prevObject = null;

            if (this.cssClasses) {
                this.$el.addClass(this.cssClasses);
            }

            if (this.toggleLabel) {
                this.toggleElement.append(this.toggleLabel);
            }

            if (this.toggleIcon) {
                this.toggleElement.addClass('fa fa-' + this.toggleIcon);
            }

            this.$el.append(
                this.toggleElement,
                this.$ul
            );

            return this;
        },

        /**
         * Update the view based on the updated properties Model
         */
        update: function () {
            var that = this;

            if (!this.model) { return; }

            _(this.model.changed).each(function (value, name) {
                var row = this.$el.find('li.dropdown-item').filter( function () {
                    return $(this).data('name') === name;
                });

                var link = row.children('a:first');
                var icon = row.find('i:first');
                var object = that.tree[name];

                var renderedRow = this.makeRow(row, link, icon, name, object.options);

                row.replaceWith(renderedRow);
            }, this);
        },

        /**
         * Option
         * @param  {Object} object      configuration
         * @return {this}               the Dropdown View
         *
         * Object can a custom field for an alternative combination active/inactie icon & label.
         *
         * Object = {
         *     name: String(),
         *     label: String(),
         *     name: String(),
         *     custom: {
         *         trueState: {
         *             label: String(),
         *             icon: String()
         *         },
         *         falseState: {
         *             label: String(),
         *             icon: String()
         *         }
         *     }
         * }
         *
         */
        option: function (object) {
            if (!object) { return; }

            this.tree[object.name] = object;

            var row  = $('<li class="dropdown-item">'),
                link = $('<a href="javascript://" class="dropdown-item__link">'),
                icon = $('<i>').addClass('fa fa-fw fa-none'),
                name = object.name,
                options = object.options;

            var renderedRow = this.makeRow(row, link, icon, name, options);

            this.$ul.append(renderedRow);

            return this;
        },

        /**
         * Divider
         */
        divider: function () {
            this.$ul.append('<li class="divider" role="separator">');
            return this;
        },

        /**
         * Header
         */
        header: function (text) {
            this.$ul.append($('<li class="dropdown-header" role="sectionhead">').text(text));
            return this;
        },

        /**
         * Make row, used in .option() and .update()
         */
        makeRow: function (row, link, icon, name, options) {
            if (!row || !link || !icon || !name) {
                return;
            }

            if (!this.tree[name]) {
                return;
            }

            options = options || {};

            var object     = this.tree[name],
                value      = this.model.get(name),
                cssClasses = object.cssClasses ? object.cssClasses : '';

            // Behaviour for custom icons
            if (this.tree[name].custom) {
                if (value) {
                    icon.attr('class', 'fa fa-' + this.tree[name].custom.trueState.icon);
                    link.text(this.tree[name].custom.trueState.label);
                }
                else {
                    icon.attr('class', 'fa fa-' + this.tree[name].custom.falseState.icon);
                    link.text(this.tree[name].custom.falseState.label);
                }
            }

            // Behaviour for default icon, using checkboxes
            else {
                if (value && !options.noIcon) {
                    icon.attr('class', 'fa fa-check');
                }
                else {
                    icon.attr('class', 'fa fa-fw fa-none');
                }
                link.text(this.tree[name].label);
            }

            // in firefox draggable=false is not enough to prevent dragging...
            if (_.device('firefox')) {
                link.attr('ondragstart', 'return false;');
            }

            // Build new row
            row.data({
                'name': name,
                'value': !value
            }).append(
                link.prepend(icon)
            );

            return row;
        },

        beforeRender: function () {

        },

        afterRender: function () {
            if (this.$el) {

                this.$el.find('.dropdown-menu').offscreen({
                    rightClass: 'right-edge',
                    leftClass: 'left-edge',
                    topClass: 'top-edge',
                    bottomClass: 'bottom-edge',
                    widthOffset: 40, //px value
                    smartResize: true
                });
            }
        }


    });

    return Dropdown;

});
