/**
/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/file/main', [
    'io.ox/files/api',
    'io.ox/core/folder/api',
    'io.ox/files/share/api',
    'io.ox/files/upload/main',
    'io.ox/core/notifications',
    'io.ox/core/capabilities',
    'settings!io.ox/core',
    'com.voiceworks/ox-messenger/file/models/file-desc',
    'com.voiceworks/ox-messenger/core/session',
    'settings!com.voiceworks/ox-messenger',
    'gettext!com.voiceworks/ox-messenger',
    'com.voiceworks/ox-messenger/aggregator'
], function (fileApi, folderApi, shareApi, fileUpload, notifications, capabilities, OXSettings, FileDesc, Session,
             Settings, gt, Aggregator) {
    'use strict';

    /**
     * File Controller
     */
    var FileController = {

        // Checks if there's space available on OX Drive
        spaceAvailable: function(){
            var properties = OXSettings.get('properties');

            return capabilities.has('infostore') &&
                (properties.infostoreQuota <= 0 || (properties.infostoreQuota - properties.infostoreUsage) > 0);
        },

        // Uploads an array of files (in OX File Format) to OX Messenger Uploads folder
        uploadFiles: function(files, callbackStartUpload, callbackEndUpload){
            var fileProcessingQueue,
                onFileUploaded = function (event, promise, fileUploadData) {
                    var exist = _.find(files, function(f){return f === fileUploadData.file;});

                    // Make sure
                    if(exist){
                        var fileDesc = new FileDesc({
                            filename: fileUploadData.file.name // (temporary) original file name
                        });

                        callbackStartUpload(fileDesc);

                        promise
                            .then(fileApi.get) // Get the uploaded file data (no need to disable cache)
                            .then(function (fileData) {
                                fileDesc.set(fileData);
                                // Just a control flag
                                fileDesc.set('upload_complete', true);
                                return fileDesc;
                            })
                            .then(callbackEndUpload);
                    }
                };

            files = _(files).map(function (file) {
                return _.extend(file, { group: 'file' });
            });

            FileController.getUploadsFolder()
                .done(function (uploadsFoler) {
                    fileProcessingQueue = fileUpload.create;
                    fileUpload.setWindowNode($('.io-ox-messenger'));
                    fileProcessingQueue.offer(files, {folder: uploadsFoler.id});

                    // Called once per file when fully uploaded
                    fileProcessingQueue.off('progress');
                    fileProcessingQueue.on('progress', onFileUploaded);

                    // whole upload progress
                    //fileUpload.collection.off('progress');
                    //fileUpload.collection.on('progress', function (baton) {
                    //    var val = Math.round(baton.progress * 100);
                    //    console.info(gt('%1$s completed', val + '%'));
                    //});
                });
        },

        // It returns the OX Messenger Uploads folder in OX format. If the folder does not exist yet
        // it will be created
        getUploadsFolder: function () {
            var that = this,
                settingsKey     = 'uploadsFolderId',
                nullFolderId    = '0',
                uploadsFolderId = Settings.get(settingsKey) || nullFolderId;

            return folderApi
                .get(uploadsFolderId)
                .then(
                    function (uploadsFolder) {
                        if (uploadsFolder.id !== nullFolderId) {
                            return uploadsFolder;

                        } else {
                            return that.createUploadsFolder();
                        }
                    },
                    // If there's an error, create a new folder
                    function(){
                        return that.createUploadsFolder();
                    })
                .then(
                    function (uploadsFolder) {
                        Settings.set(settingsKey, uploadsFolder.id);
                        Settings.save();
                        return uploadsFolder;
                    },
                    function(){
                        notifications.yell('error', gt('There was a problem accessing the uploads folder'));
                    }
                );
        },

        // It creates OX Messenger Uploads folder and returns an OX folder object
        createUploadsFolder: function(){
            var oxDrive = require('settings!io.ox/core').get('folder/infostore');
            return folderApi.create(
                oxDrive,
                {
                    title: gt('OX Messenger Uploads')
                }
            );
        },

        // It will share a file with 'READ' permissions with a list of users
        // It returns a promise with the updated file descriptor
        shareFile: function(fileDesc, users) {
            var that = this,
                anyGuest = false,
                objectPermissions = [],
                fileObj,
                options,
                changes;

            // File object/data to be sent
            fileObj = fileDesc.pick('id', 'folder_id');

            // Remove logged user from the list
            users = users.filter(function (user) {
                return user.id !== Session.user.id;
            });

            // Create the OX users' permissions array (skip guests)
            users.forEach(function (user) {
                if(!user.isGuest()){
                    // It might be the case a guest user enters this block due to
                    // provisioning expiration. We can detect that by checking
                    // the contact.id
                    if(user.get('contact').id){
                          objectPermissions.push({
                            bits   : 1,// 1:view permissions, 2:write permissions,
                            entity : user.get('contact').id,
                            group  : false
                        });
                    }
                }else{
                    anyGuest = true;
                }
            });

            // Create list of users to share the file with
            changes = {object_permissions: objectPermissions};

            // Permission options
            options = {
                cascadePermissions : false,
                notification       : false // {transport: 'mail'} in case we want mail notifications
            };

            return fileApi
                    .update(fileObj, changes, options)
                    .then(
                        function(){
                            // If there's any guest user, we have to create a public LINK
                            if(anyGuest)
                                return that.createFilePublicLink(fileDesc).then(
                                    function(publicLink) {
                                        // Update fileDesc with the public url
                                        fileDesc.set('public_link', publicLink);
                                        return fileDesc;
                                    }
                                );
                            return fileDesc;
                        }
                    )
                    .then(null, function(e){
                        notifications.yell('error', e.error);
                    });
        },

        // Returns a promise with the full file Id ('folder_id/file_id')
        cancelShareFile: function(fileDesc) {
            var fileObj,
                options,
                changes;

            // File object/data to be sent
            fileObj = fileDesc.pick('id', 'folder_id');

            // Create list of users to share the file with
            changes = {object_permissions: []};

            // Permission options
            options = {
                cascadePermissions : false,
                notification       : false // {transport: 'mail'} in case we want mail notifications
            };

            // This will remove also public links
            return fileApi.update(fileObj, changes, options);
        },

        // It returns a promise with the public link
        createFilePublicLink: function(fileDesc) {
            var data = {
                module: 'infostore',
                folder: fileDesc.get('folder_id'),
                item: fileDesc.get('id')
            };

            return shareApi
                .getLink(data)
                .then(function(data){
                    return data.url;
                });
        },

        // It returns the full OX file data
        getFileData: function(fileDesc){
            return fileApi
                .get(fileDesc.toObject(true), {cache: false});
        },

        // OX model/format. Used to pop up file permissions
        getFileShareData: function(fileDesc){
            var fileData = fileDesc.pick('id', 'folder_id'),
                options = {cache: false}; // Important!!

            return shareApi.getFileShare(fileData, options)
                .then(function (data) {
                    return new shareApi.Model(data);
                });
        },

        // It returns a promise with the full OX file data
        copyFileToUploads: function(fileDesc) {
            return this
                // Get Uploads Folder
                .getUploadsFolder()
                // Copy the file into the Folder
                .then(function(folderData){
                    return fileApi.copy([fileDesc.toObject()], folderData.id, true);
                })
                // Get the full file data
                .then(function(filesPartialData){
                    var fileFullId = filesPartialData[0].data,
                        folderId = fileFullId.substring(0, fileFullId.indexOf('/')),
                        fileId = fileFullId.substring(fileFullId.indexOf('/') + 1);

                    return fileApi.get(
                        {folder_id: folderId, id: fileId},
                        {cache: false}
                    );
                })
                // Convert the file data into a file Descriptor
                .then(function(fileData){
                    return new FileDesc(fileData);
                });
        },

        patchOXFileDeepLinks: function () {
            $(document).on('click', '.deep-link-files-oxm', function (e) {
                var folderId = $(this).attr('folder-id');
                var fileId = $(this).attr('file-id');

                if (fileId) {
                    // open file in viewer
                    require(['io.ox/core/viewer/main', 'io.ox/files/api'], function (Viewer, api) {
                        api.get({folder: folderId, id: fileId}).then(
                            function sucess(data) {
                                var viewer = new Viewer();
                                viewer.launch({ files: [data] });
                            }
                        );
                    });
                } else {
                    // open folder in viewer
                    require(['io.ox/core/folder/api'], function (api) {
                        api.get(folderId).then(
                            function () {
                                ox.launch('io.ox/files/main', { folder: folderId}).done(function () {
                                    // set proper folder
                                    if (this.folder.get() !== folderId) this.folder.set(folderId);
                                });
                            }
                        );
                    });
                }
                e.preventDefault();
            });
        }
    };


    return FileController;
});
