/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/main', [
    'io.ox/core/api/user',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/roster/views/roster',
    'com.voiceworks/ox-messenger/roster/models/groupcollection',
    'com.voiceworks/ox-messenger/roster/models/group',
    'com.voiceworks/ox-messenger/roster/models/usercollection',
    'com.voiceworks/ox-messenger/core/user',
    'com.voiceworks/ox-messenger/core/views/partials/userpart',
    'com.voiceworks/ox-messenger/util',
    'gettext!com.voiceworks/ox-messenger',
    'less!com.voiceworks/ox-messenger/style',
    'less!com.voiceworks/ox-messenger/phone/style'
], function (userApi, Aggregator, RosterView, RosterGroupCollection, RosterGroupModel, RosterUserCollection, UserModel,
             UserPart, Util, gt) {
    'use strict';

    /**
     * Main Roster Application
     */
    var Roster = {
        init: function (user) {
            if (this.created) {
                return;
            }

            // Remove old toggle listener
            Aggregator.off('roster:toggle');
            Aggregator.on('roster:toggle', $.proxy(this.toggleHandler, this));
            Aggregator.on('session:disconnected', $.proxy(this.offlineHandler, this));
            Aggregator.on('roster:hide', $.proxy(this.hide, this));

            this.user = user;
            this.groups = new RosterGroupCollection();
            this.created = false;
            this.maxCustomGroups = 10;
            // To store those statuses received before the roster was created
            this.storedStatuses = [];
            // To avoid sorting multiple times in a time lapse
            this.sortTimeoutId = null;

            // Create default Buddylist group
            var buddylist = new RosterGroupModel({
                id: 1,
                name: gt('Buddylist'),
                index: 9000 // It's over 9000! Put at bottom
            });

            var favorites = new RosterGroupModel({
                id: 2,
                name: gt('Favorites'),
                index: 100 // Put at the top
            });

            var guestUsers = new RosterGroupModel({
                id: 3,
                name: gt('Guest Users'),
                index: 150 // Put below Favorites
            });

            this.groups.add([buddylist, favorites, guestUsers]);

            require('io.ox/core/notifications').on('show', $.proxy(this.hide, this));

            this.created = true;
        },

        attach: function () {
            $('#io-ox-core').prepend(
                $('<div class="messenger--roster messenger--is-hidden" tabindex="-1">'),
                $('<div class="abs messenger--roster-overlay messenger--is-hidden">').click($.proxy(this.hide, this))
            );
        },

        hasUser: function (id) {
            var theSession = require('com.voiceworks/ox-messenger/core/session');

            id = Util.getBareJid(id);

            if (Util.isSameJid(theSession.user.id, id)) {
                return theSession.user;
            }

            return this.getAllUsers().get(id);
        },

        getUserById: function (id) {
            // @TODO refactor this, just refactor handleMessage from sessionuser
            var Session = require('com.voiceworks/ox-messenger/core/session');

            id = Util.getBareJid(id);

            if (Util.isSameJid(Session.user.id, id)) {
                return Session.user;
            }

            return this.getAllUsers().get(id) || new UserModel({
                    id: id,
                    inRoster: false
                });
        },

        getAllUsers: function () {
            return this.groups.get(1).users;
        },

        getNextUnusedId: function () {
            // It will never reach 200
            for(var i = 103;i<200;i++) {
              if(!this.groups.get(i))
                return i;
            }

            return -1;
        },

        getGroups: function () {
            return this.groups;
        },

        getGroup: function (id) {
            return this.groups.get(id);
        },

        getGroupsObject: function () {
            if (!this.groups) {
                return;
            }

            var object = {};

            this.groups.each(function (group) {
                var groupId = Util.keyify(group.id);
                object[groupId] = {
                    name: group.get('name'),
                    toggled: group.get('toggled')
                };
            });

            return object;
        },

        getGroupKeys: function () {
            var object = this.getGroupsObject();

            if (!object) {
                return;
            }

            var objectKeys =  _.keys(object);
            var objectKeysClean = _.map(objectKeys, function(key) {
                return key.toLowerCase().replace(/\W/g, '_');
            });

            return objectKeysClean;
        },

        getGroupValues: function () {
            var object = this.getGroupsObject();

            if (!object) {
                return;
            }

            return _.values(object);
        },

        getGroupToggleStatuses: function () {
            var groups              = this.getGroups(),
                groupToggleStatuses = {};

            groups.each(function (group) {
                groupToggleStatuses[group.id] = group.get('toggled');
            });

            return groupToggleStatuses;
        },

        getGroupUsers: function (id) {
            var group = this.groups.get(id);

            if (!group) {
                return null;
            }

            return group.users;
        },

        create: function (buddies) {
            var that            = this,
                contacts        = this.user.contacts,
                usersObj        = {};

            // Create empty user arrays for the groups
            that.groups.each(function(groupId) {
                usersObj[groupId.get('id')] = [];
            });

            // Build Roster
            _.each(buddies, function (buddy, jid) {
                var user    = that.getUserById(jid),
                    contact = _.find(contacts, function (contact) {
                        if (contact.email1)
                            return contact.email1.toLowerCase() === jid;
                    });

                // Update user model
                user.set(buddy);
                user.set({
                    id: jid,
                    contact: contact || {},
                    inRoster: true
                });

                // Set subscription
                // @QUESTION: What is subscription?
                if (user.get('subscription') !== buddy.subscription) {
                    user.set({
                        'subscription': buddy.subscription
                    });
                }

                // Add all users to the Buddylist
                usersObj[1].push(user);

                UserPart.updatePresenceBadges(jid, 'offline');

                // Check for groups
                if (buddy.groups.length === 0) {
                    return;
                }

                // Add users to groups
                _.each(buddy.groups, function (groupId) {
                    // Add the groupId to the Properties of this User
                    user.properties.addProperty(groupId, true);

                    if(!usersObj[groupId]) {
                        usersObj[groupId] = [];
                    }
                    usersObj[groupId].push(user);
                });
            });

            _.each(usersObj, function (users, groupId) {
                var group = that.groups.get(parseInt(groupId, 10));

                // Data might be corrupted
                if(group) {
                    group.users.reset(users);
                }
            });

            // Create the view at the end. After processing all data
            if(!this.view) {
                this.view = new RosterView({ user: this.user, collection: this.groups });
            }

            // Enable/disable 'AddNewGroup' feature
            this.limitCustomGroups();

            // Process the statuses we tried to set before the roster was created, and reset it
            this.processStoredStatuses();

            Aggregator.trigger('roster:created', true);
        },

        createGroup: function (groupName, id, index) {
            var id        = id ? id : this.getNextUnusedId(),
                index     = index ? index : 200;

            if (!id) {
                return;
            }

            var group = new RosterGroupModel({
                id: id,
                name: groupName,
                index: index
            });

            this.groups.add(group);

            return group;
        },

        updateGroup: function (group, name) {
            if (!group) {
                return;
            }

            group.set({
                name: name
            });

            return group;
        },

        deleteGroup: function (id) {
            var group = this.getGroup(id);

            if (group) {
                this.groups.remove(group);
            }

            return group;
        },

        setStatus: function (statusObj, avoidGroupSort) {
            var id = statusObj.username,
                statuses = statusObj.status || {},
                buddyStatus = { status: 'offline' },
                buddyMessage = '',
                buddylist = this.groups.get(1),
                users = buddylist.users,
                sortOn = [ 'invisible', 'offline' ],
                existingUser = users.get(id),
                nickname = '',
                presenceBadge;

            // If the user doesn't exit, we store the status to render it later
            if (!existingUser) {
                this.storedStatuses.push(statusObj);
                return;
            }

            var prevStatus = existingUser.get('status');
            if (prevStatus === statusObj.status) {
                return;
            }

            _.each(statuses, function (statusObj, resource) {
                if (statusObj.status === 'offline') {
                    buddyStatus = statusObj;
                    nickname = statusObj;
                }

                //if (resource.match(/\/oxmessenger$/)) {
                buddyStatus = statusObj;
                buddyMessage = statusObj.message;
                nickname = statusObj.nickname;
                //}

                if (!buddyStatus.capabilities) {
                    buddyStatus.capabilities.chat = true;
                    buddyStatus.capabilities.audio = false;
                    buddyStatus.capabilities.video = false;
                    buddyStatus.capabilities.mustFlash = false;
                    buddyStatus.capabilities.hasFlash = false;
                }
            });

            existingUser.set({
                canChat: buddyStatus.capabilities.chat,
                canAudio: buddyStatus.capabilities.audio,
                canVideo: buddyStatus.capabilities.video,
                mustFlash: buddyStatus.capabilities.mustFlash,
                hasFlash: buddyStatus.capabilities.hasFlash,
                priority: buddyStatus.priority,
                statusMessage: buddyStatus.message,
                nickname: nickname
            });

            existingUser.setStatus(buddyStatus.status, buddyStatus.message);

            // Only sort the list when a user comes or goes offline/invisible
            if (sortOn.indexOf(prevStatus) !== -1 || sortOn.indexOf(buddyStatus.status) !== -1) {
                if(!avoidGroupSort)
                    this.sortGroups();
            }

            // find drawn presence extension points and update the class
            if (buddyStatus.status === 'invisible') {
                UserPart.updatePresenceBadges(id, 'offline');
            } else {
                UserPart.updatePresenceBadges(id, buddyStatus.status);
            }
        },

        sortGroups: function() {
            var that = this;
            var interval = 100;

            clearTimeout(this.sortTimeoutId);
            this.sortTimeoutId = setTimeout(function(){
                that.groups.each(function (group) {
                    group.users.sort();
                });
            }, interval);
        },

        processStoredStatuses: function() {
            var that = this;
            
            this.storedStatuses.forEach(function(statusObj) {
                var avoidGroupSort = true;
                that.setStatus(statusObj, avoidGroupSort);
            });
            this.storedStatuses = [];
            this.sortGroups();
        },

        updatePresenceBadges: function (jid, status) {
            $('.messenger--presence[data-jid="' + jid + '"]')
                .removeClass('is-status--unknown ' +
                'is-status--offline ' +
                'is-status--online ' +
                'is-status--away ' +
                'is-status--dnd ' +
                'is-status--invisible ' +
                'is-status--xa')
                .addClass('is-status--' + status);
        },

        toggleHandler: function (event) {
            this.view.$el.css({
                    'top': $('#io-ox-banner:visible').outerHeight() + $('#io-ox-topbar:visible').outerHeight()
                })
                .toggleClass('messenger--is-hidden')
                .siblings('.messenger--roster-overlay').toggleClass('messenger--is-hidden');

            if (this.view.$el.is(':visible')) { // focus on search when roster is visible
                require('io.ox/core/notifications').hide();
                $('#roster-search').focus();
            }
        },


        /**
         * TODO this should be handled by the status change, not manually
         */
        offlineHandler: function (event) {
            // Force al roster users to offline
            var allUsers = this.getAllUsers();
            allUsers.each(function (user) {
                user.setStatus('offline');
            });
        },


        show: function () {
            this.view.$el.removeClass('messenger--is-hidden')
                .siblings('.messenger--roster-overlay').removeClass('messenger--is-hidden');
        },

        hide: function () {
            if (this.view) {
                this.view.$el.addClass('messenger--is-hidden')
                    .siblings('.messenger--roster-overlay').addClass('messenger--is-hidden');
            }
        },

        limitCustomGroups: function() {
            if (this.groups.models.length > (this.maxCustomGroups + 2)) {
                this.view.addNewGroup.show(false);
            }
            else {
                this.view.addNewGroup.show(true);
            }
        }
    };

    return Roster;
});
